require File.dirname(__FILE__) + '/spec_helper'

require 'useless'

describe 'museum.useless.io' do
  include Rack::Test::Methods

  def app
    Useless::Rack.new do
      map 'museum' => Useless::Museum
    end
  end

  describe 'GET /photos/:id' do
    it 'should return the record corresponding to the specified ID' do
      shot_at = Time.now - (24 * 60 * 60)
      created_at = Time.now
      user_id = Useless.mongo['users'] << {handle: 'khy'}
      photo_id = Useless.mongo['museum.photos'] << {
        file_url: {
          small: 'http://museum.useless.io/files/abc123',
          medium: 'http://museum.useless.io/files/def456',
          large: 'http://museum.useless.io/files/ghi789'
        },
        latitude: 71.567,
        longitude: 48.987,
        shot_at: shot_at,
        description: 'An image of the Scientology headquarters',
        created_by_id: user_id,
        created_at: created_at,
        updated_at: created_at
      }

      get "http://museum.useless.io/photos/#{photo_id}"
      last_response.should be_ok

      body = JSON.parse(last_response.body)
      body['id'].should == photo_id.to_s
      body['file_url']['small'].should == 'http://museum.useless.io/files/abc123'
      body['file_url']['medium'].should == 'http://museum.useless.io/files/def456'
      body['file_url']['large'].should == 'http://museum.useless.io/files/ghi789'
      body['latitude'].should == 71.567
      body['longitude'].should == 48.987
      body['shot_at'].should == shot_at.utc.iso8601
      body['description'].should == 'An image of the Scientology headquarters'
      body['created_by']['id'].should == user_id.to_s
      body['created_by']['handle'].should == 'khy'
      body['created_at'].should == created_at.utc.iso8601
      body['updated_at'].should == created_at.utc.iso8601
    end

    it 'should return a 404 Not Found if there is no record corresponding to the specified ID' do
      get "http://museum.useless.io/photos/#{BSON::ObjectId.new.to_s}"
      last_response.should be_not_found
    end

    it 'should return a 404 Not Found if the specified ID is not a valid BSON::ObjectId' do
      get "http://museum.useless.io/photos/invalid-bson-object-id"
      last_response.should be_not_found
    end
  end

  describe 'POST /photos' do
    it 'should create a new photo record if latitude, longitude and shot_at are not specified,
        but can be extracted from the file and the request is authenticated' do
      user_id = Useless.mongo['users'] << {handle: 'khy', access_token: 'abc123'}

      header 'Authorization', 'abc123'
      post 'http://museum.useless.io/photos',
        file: upload_file('muffin-milk.jpg', 'image/jpeg'),
        description: 'The Muffin Milk Man!'

      last_response.status.should == 201
      get last_response.header['Location']
      last_response.should be_ok

      body = JSON.parse(last_response.body)
      body['latitude'].to_i.should == 40
      body['longitude'].to_i.should == -73
      body['shot_at'].should == '2011-11-14T02:23:42Z'
      body['description'].should == 'The Muffin Milk Man!'
      body['created_by']['id'].should == user_id.to_s
      body['created_by']['handle'].should == 'khy'
      Time.parse(body['created_at']).should be_within(2).of(Time.now)
      Time.parse(body['updated_at']).should be_within(2).of(Time.now)

      get body['file_url']['small']
      image = MiniMagick::Image.read(last_response.body)
      [image[:width], image[:height]].should include(100)

      get body['file_url']['medium']
      image = MiniMagick::Image.read(last_response.body)
      [image[:width], image[:height]].should include(500)

      get body['file_url']['large']
      image = MiniMagick::Image.read(last_response.body)
      [image[:width], image[:height]].should include(1024)
    end

    it 'should respond with a 401 Unauthorized if no access code is specified' do
      post 'http://museum.useless.io/photos',
        file: upload_file('muffin-milk.jpg', 'image/jpeg'),
        description: 'The Muffin Milk Man!'
      last_response.status.should == 401
      last_response.body.should == 'The request could not be authenticated.'
    end

    it 'should respond with a 422 Unprocessible Entry if the latitude, longitude and shot at are
        not specified and cannot be extracted' do
      user_id = Useless.mongo['users'] << {handle: 'khy', access_token: 'abc123'}

      header 'Authorization', 'abc123'
      post 'http://museum.useless.io/photos',
        file: upload_file('muffin-milk-stripped.jpg', 'image/jpeg'),
        description: 'The Muffin Milk Man!'
      last_response.status.should == 422
      last_response.body.should == 'latitude, longitude or shot_at could not be extracted from the photo. Retry with the latitude, longitude and shot_at attributes specified.'
    end

    it 'should respond with 201 if latitude, longitude and shot_at cannot be extracted but are specified' do
      user_id = Useless.mongo['users'] << {handle: 'khy', access_token: 'abc123'}

      header 'Authorization', 'abc123'
      post 'http://museum.useless.io/photos',
        file: upload_file('muffin-milk-stripped.jpg', 'image/jpeg'),
        description: 'The Muffin Milk Man!',
        latitude: 71.567,
        longitude: 48.987,
        shot_at: (Time.now - (24 * 60 * 60)).utc.iso8601

      last_response.status.should == 201
      get last_response.header['Location']
      last_response.should be_ok
    end

    it 'should respond with a 422 if the file type is invalid, even if latitude, longitude and shot at are specified' do
      user_id = Useless.mongo['users'] << {handle: 'khy', access_token: 'abc123'}

      header 'Authorization', 'abc123'
      post 'http://museum.useless.io/photos',
        file: upload_file('muffin-milk.txt', 'text/plain'),
        latitude: 71.567,
        longitude: 48.987,
        shot_at: (Time.now - (24 * 60 * 60)).utc.iso8601

      last_response.status.should == 422
      last_response.body.should == 'Invalid image format. Format must be one of JPEG, TIFF, PNG or GIF.'
    end
  end
end
